-- LGPL v2.1+

local function is_filling_node(name)
	return name == "air" or name == "default:water_source"
end


minetest.register_node("unknownburn:fire", {
	drawtype = "firelike",
	tiles = {
		{
			name = "fire_basic_flame_animated.png^[colorize:blue:127",
			animation = {
				type = "vertical_frames",
				aspect_w = 16,
				aspect_h = 16,
				length = 1,
			},
		},
	},
	inventory_image = "fire_basic_flame.png^[colorize:blue:127",
	paramtype = "light",
	light_source = 14,
	walkable = false,
	buildable_to = true,
	sunlight_propagates = true,
	groups = {dig_immediate = 3, not_in_creative_inventory = 1},
})

minetest.register_tool("unknownburn:flint", {
	description = "Unknown Nodes flint",
	inventory_image = "fire_flint_steel.png^[colorize:blue:127",
	on_use = function(itemstack, user, pointed_thing)
		if pointed_thing.type ~= "node" then return end
		if not minetest.registered_nodes[minetest.get_node(pointed_thing.under).name] then
			minetest.set_node(pointed_thing.under, {name = "unknownburn:fire"})
		end
	end
})

minetest.register_abm({
	nodenames = {"unknownburn:fire"},
	interval = 1,
	chance = 10,
	catch_up = false,
	action = function(pos, node, active_object_count, active_object_count_wider)

		-- Check the enviroment and set fire
		local known_count = 0
		local unknown_count = 0
		local ignored_count = 0
		local burn_count = 0
		local filler_count = 0
		local nodenames = {}
		local unknowns = {}
		for x = pos.x-3, pos.x+3 do
			for y = pos.y-3, pos.y+3 do
				for z = pos.z-3, pos.z+3 do --7x7x7 = 343 nodes checked
					local name = minetest.get_node({x=x, y=y, z=z}).name
					if name == 'ignore' then
						ignored_count = ignored_count + 1
					elseif name == "unknownburn:fire" then
						burn_count = burn_count + 1
					elseif minetest.registered_nodes[name] then
						known_count = known_count + 1
						local value
						if pos.y == y then -- prefer node type from same hight level
							value = (10 - (math.abs(pos.x-x) + math.abs(pos.z-z)))*5
						else
							value = 10-(math.abs(pos.x-x)+math.abs(pos.y-y)+math.abs(pos.z-z))
						end
						if not nodenames[name] then
							nodenames[name] = value
						else
							nodenames[name] = nodenames[name] + value
						end
						if is_filling_node(name) then
							filler_count = filler_count + 1
						end
					else
						unknown_count = unknown_count + 1
						table.insert(unknowns, {x=x, y=y, z=z})
					end
				end
			end
		end
		if burn_count < 30 and #unknowns > 0 then -- maximum count
			local newburn = math.random(#unknowns)
			minetest.set_node(unknowns[newburn], {name = "unknownburn:fire"})
			local newburn = math.random(#unknowns)
			minetest.set_node(unknowns[newburn], {name = "unknownburn:fire"})
		end

		--print("known", known_count, "unknown",unknown_count, "burn", burn_count, "filler", filler_count, dump(nodenames))

		-- Checks if the current fire node should removed
		if ignored_count > 0 or known_count < ( unknown_count+burn_count)*2 then
			return
		end

		-- Calculate the replacement node to slake fire
		local selected = {}
		-- usually there is more air as other (different) nodes. So there is a special handling for air. The same for water
		if filler_count > 170 then --~1/2 of 343
			if nodenames["default:water_source"] and not nodenames["air"] then
				selected.name = "default:water_source"
			elseif not nodenames["default:water_source"] and nodenames["air"] then
				selected.name = "air"
			elseif nodenames["default:water_source"] > nodenames["air"] then
				selected.name = "default:water_source"
			else
				selected.name = "air"
			end
		else
			for name, count in pairs(nodenames) do
				if is_filling_node(name) then
					--ignore
				elseif not selected.name or selected.count < count then
					selected.name = name
					selected.count = count
				end
			end
		end

		--print("placing",  selected.name, "to", minetest.pos_to_string(pos))
		if selected.name then
			minetest.add_node(pos, {name = selected.name})
		end
	end
})
